﻿using Cfg.Attributes;
using Cfg.ConfigProviders;
using NUnit.Framework;

namespace Cfg.Test.Attributes
{
    [TestFixture]
    public class MapAttributeTest
    {
        public interface ITestConfig
        {
            string X { get; } 

            string Var { get; }

            IFooSection FooSection { get; }
        }

        public interface IFooSection
        {
            [Map("Bar")]
            string Bar { get; }
        }

        public class FakeProvider : IConfigFileProvider
        {
            public ConfigKeyValues GetValues()
            {
                return new ConfigKeyValues()
                    .Add("x", "x")
                    .Add("bar", "baaar")
                    .Add("var", "%bar%/var");
            }
        }

        [Test]
        public void MapFlatFileToSections()
        {
            var configFileProvider = new FakeProvider();
            AppSettings.Init<ITestConfig>(configFileProvider);

            var cfg = AppSettings.Get<ITestConfig>();

            Assert.AreEqual("x", cfg.X);
            Assert.AreEqual("bar", cfg.FooSection.Bar);
        }

        [Test]
        public void UseMappedValuesAsVariable()
        {
            var configFileProvider = new FakeProvider();
            AppSettings.Init<ITestConfig>(configFileProvider);

            var cfg = AppSettings.Get<ITestConfig>();

            Assert.AreEqual("baaar/var", cfg.Var);
        }
    }
}
