using System;
using System.Security.Cryptography.X509Certificates;
using Cfg.Attributes;
using Cfg.ConfigValues;
using NUnit.Framework;
using NUnit.Mocks;

namespace Cfg.Test
{
	[TestFixture]
	public class PathTest
	{
        public interface ITestConfig
        {
            PathInfo PathInfo { get; set; } 
        }

        [Test]
	    public void Test()
	    {
	        Cfg.AppSettings.Init<ITestConfig>();
	    }


		[Test]
		public void GetRelativePathFromAbsolute()
		{
			var attr = new object[] { new PathAttribute(false, false) };

			var absolutePath = AppDomain.CurrentDomain.BaseDirectory.Replace("\\Release", "\\xxx").Replace("\\Debug", "\\xxx") + "\\x";
			Console.WriteLine(absolutePath);

			var vb = new ValueBuilder(typeof(PathInfo), attr, null, null, absolutePath);
			var value = (PathInfo)vb.GetValue();

			Assert.AreEqual("..\\xxx\\x", value.RelativePath);
		}

		[Test]
		public void GetAbsolutePathFromRelative()
		{
			var attr = new object[] { new PathAttribute(false, false) };

			var relativePath = "x/y/z";
			var vb = new ValueBuilder(typeof(PathInfo), attr, null, null, relativePath);
			var value = (PathInfo)vb.GetValue();

			var absolutePath = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, relativePath.Replace(System.IO.Path.AltDirectorySeparatorChar, System.IO.Path.DirectorySeparatorChar));
			Console.WriteLine(absolutePath);

			Assert.AreEqual(absolutePath, value.FullPath);
		}
	}
}